import { ParamValueMode } from "jalhyd";

import { ParamRadioConfig, NgParameter } from "../ngparam";
import { FormulaireDefinition } from "./form-definition";

/**
 * gestion des formulaires avec "paramètre fixé" et "paramètre à varier"
 */
export class FormDefFixedVar {
    protected _formBase: FormulaireDefinition;

    constructor(base: FormulaireDefinition) {
        this._formBase = base;
    }

    /**
     * remet les radios de tous les paramètres à FIX sauf "me" et ceux (celui) à l'état "except"
     */
    protected resetOtherRadio(me: NgParameter, except?: ParamRadioConfig) {
        for (const p of this._formBase.allFormElements) {
            if (p instanceof NgParameter) {
                if (p !== me && p.radioState !== except
                    && p.radioState !== ParamRadioConfig.LINK
                    && p.radioConfig !== ParamRadioConfig.FIX) {

                    p.valueMode = ParamValueMode.SINGLE;
                }
            }
        }
    }

    /**
     * gère un changement de mode pour un paramètre
     * règles :
     *   - 1 seul paramètre CAL à la fois
     *   - 1 seul paramètre multivalué (VAR) à la fois (directement ou par liaison)
     *   - plusieurs paramètres FIX à la fois possible
     *   - plusieurs paramètres LINK à la fois possible si les 2 1ères règles sont respectées
     *
     * analyse :
     * ancien état    nouvel état    action(s)
     * FIX            VAR            action1
     * FIX            CAL            action2
     * FIX            LINK           si paramètre lié FIX : aucune
     *                               si paramètre lié VAR : action1
     *                               si paramètre lié CAL : si valeur unique : aucune
     *                                                      si valeur multiple : action1
     *                               si paramètre lié LINK : recommencer ce cas avec le paramètre lié
     *
     * VAR            FIX            aucune
     * VAR            CAL            action2
     * VAR            LINK           si paramètre lié FIX : aucune
     *                               si paramètre lié VAR : aucune
     *                               si paramètre lié CAL : si valeur unique : aucune
     *                                                      si valeur multiple : aucune
     *                               si paramètre lié LINK : recommencer ce cas avec le paramètre lié
     *
     * CAL            FIX            action5
     * CAL            VAR            action3 + action5
     * CAL            LINK           si paramètre lié FIX : aucune
     *                               si paramètre lié VAR : action3 + action4|action5
     *                               si paramètre lié CAL : action3 + action4|action5
     *                               si paramètre lié LINK : recommencer ce cas avec le paramètre lié
     *
     * action1 : reset (à FIX) de tous les autres paramètres que celui modifié sauf celui à CAL
     * action2 : reset (à FIX) de tous les autres paramètres que celui modifié sauf celui/ceux à VAR
     * action3 : reset (à FIX) de tous les autres paramètres que celui modifié
     * action4 : mettre le paramètre désigné par la conf comme "par défault" à CAL
     * action5 : mettre le 1er paramètre du module de calcul à CAL
     */
    protected processRadioStateChange(sourceParam: NgParameter, oldState: ParamValueMode) {
        switch (oldState) {
            case ParamValueMode.SINGLE:  // ancien état
                switch (sourceParam.valueMode) {
                    case ParamValueMode.MINMAX:  // nouvel état
                    case ParamValueMode.LISTE:
                        this.resetOtherRadio(sourceParam, ParamRadioConfig.CAL);
                        break;

                    case ParamValueMode.CALCUL:  // nouvel état
                        this.resetOtherRadio(sourceParam, ParamRadioConfig.VAR);
                        break;

                    case ParamValueMode.LINK:  // nouvel état
                        if (sourceParam.paramDefinition.hasMultipleValues) {
                            this.resetOtherRadio(sourceParam, ParamRadioConfig.CAL);
                        } else {
                            const refParamValues = sourceParam.paramDefinition.referencedParamValues;
                            if (refParamValues !== undefined) { // cad si on référence un paramètre et non un Result par ex
                                if (refParamValues.valueMode === ParamValueMode.LINK) {
                                    throw new Error(`références de paramètre en chaîne non pris en charge`);
                                }
                            } // cas à traiter
                        }
                        break;
                }
                break;

            case ParamValueMode.LISTE:  // ancien état
            case ParamValueMode.MINMAX:
                switch (sourceParam.valueMode) {
                    case ParamValueMode.CALCUL:  // nouvel état
                        this.resetOtherRadio(sourceParam, ParamRadioConfig.VAR);
                        break;

                    case ParamValueMode.LINK:  // nouvel état
                        // mode du paramètre référencé
                        const refParamValues = sourceParam.paramDefinition.referencedParamValues;
                        if (refParamValues.valueMode === ParamValueMode.LINK) {
                            throw new Error(`références de paramètre en chaîne non pris en charge`);
                        }
                        break;
                }
                break;

            case ParamValueMode.LINK:  // ancien état
                switch (sourceParam.valueMode) {
                    case ParamValueMode.MINMAX:  // nouvel état
                    case ParamValueMode.LISTE:
                        this.resetOtherRadio(sourceParam, ParamRadioConfig.CAL);
                        break;

                    case ParamValueMode.CALCUL:  // nouvel état
                        this.resetOtherRadio(sourceParam, ParamRadioConfig.VAR);
                        break;
                }
                break;
        }
    }

    /**
     * modifie les boutons radio "fix", "var", "cal" de tous les paramètres
     * en fonction de la modification de l'état d'un des paramètres
     * @param uid id numérique unique du paramètre source
     * @param option nouvel état "fix", "var" ou "cal" du paramètre source
     */
    protected resetRadiosAndResults(sourceParam: NgParameter, oldState: ParamValueMode) {
        this.processRadioStateChange(sourceParam, oldState);

        // on vérifie qu'il y a au moins un paramètre "à calculer" et sinon, on prend le 1er qui est à "fixé"
        if (this._formBase.getDisplayedParamFromState(ParamRadioConfig.CAL) === undefined) {
            let newCal: NgParameter;

            for (const p of this._formBase.allFormElements) {
                if (p instanceof NgParameter) {
                    // change all radio button groups except the one that sent the event
                    if (p.radioConfig === ParamRadioConfig.CAL && p.radioState === ParamRadioConfig.FIX && p !== sourceParam) {
                        newCal = p;
                        break;
                    }
                }
                if (newCal) {
                    break;
                }
            }
            // if the current calculated parameter was set to another mode, set a new param
            // to calculated mode (there must always be at least one)
            if (newCal) {
                newCal.valueMode = ParamValueMode.CALCUL;
            }
        }
    }

    private logParams() {
        for (const fe of this._formBase.allFormElements) {
            if (fe instanceof NgParameter) {
                console.log(`${fe.paramDefinition.symbol} : ${ParamValueMode[fe.paramDefinition.valueMode]}`);
            }
        }
    }

    /**
     * gestion des événements clic sur les radios
     */
    public onRadioClick(info: any) {
        const param: NgParameter = info.param; // paramètre source de l'événement radio
        const old: ParamValueMode = info.oldValueMode; // ancien état (radio)
        this.resetRadiosAndResults(param, old);
    }
}
